@@ -1,7 +1,10 @@
+#include "../common/global.h"
 #include "udev.h"
 #include "global.h"
 #include "cpu.h"
 
+#define _PATH_DEVTREE          "/proc/device-tree/compatible"
+
 // https://www.kernel.org/doc/html/latest/core-api/cpu_hotplug.html
 int get_ncores_from_cpuinfo(void) {
   // Examples:
@@ -143,8 +146,7 @@ char* get_field_from_cpuinfo(char* CPUINFO_FIELD) {
   char* tmp2 = strstr(tmp1, "\n");
 
   int strlen = (1 + (tmp2-tmp1));
-  char* hardware = emalloc(sizeof(char) * strlen);
-  memset(hardware, 0, sizeof(char) * strlen);
+  char* hardware = ecalloc(strlen, sizeof(char));
   strncpy(hardware, tmp1, tmp2-tmp1);
 
   return hardware;
@@ -349,3 +351,79 @@ bool is_devtree_compatible(char* str) {
   }
   return true;
 }
+
+char* get_devtree_compatible(int *filelen) {
+  char* buf;
+
+  if ((buf = read_file(_PATH_DEVTREE, filelen)) == NULL) {
+    printWarn("read_file: %s: %s", _PATH_DEVTREE, strerror(errno));
+  }
+
+  return buf;
+}
+
+// Returns a list of structs devtree, each containing both the vendor and the
+// model, coming from the compatible file from the device tree. In this
+// context, vendor refers to the first string of every entry and the model to
+// the second. For instance, given a compatible file with:
+// "str1,foo1.str2,foo2" (where . denotes the NULL byte, i.e., the separator),
+// then this function will return a list with two structs, the first one
+// containing str1 and foo1 and the other containing str2 and foo2.
+struct devtree** get_devtree_compatible_struct(int *num_vendors_ptr) {
+  int len;
+  char* dt = get_devtree_compatible(&len);
+  if (dt == NULL) {
+    return NULL;
+  }
+
+  int num_vendors = 0;
+  char* ptr = dt;
+
+  for (int ptrpos = 0; ptrpos < len; ptrpos = (ptr-dt)) {
+    ptr = memchr(ptr, '\0', len);
+    if (ptr == NULL) {
+      printBug("get_devtree_compatible_struct: Unable to find delimiter (1) (num_vendors=%d)", num_vendors);
+      return NULL;
+    }
+    ptr++;
+    num_vendors++;
+  }
+
+  struct devtree** vendors = emalloc(sizeof(struct devtree *) * num_vendors);
+  ptr = dt;
+
+  for (int ptrpos = 0, i = 0; ptrpos < len; ptrpos = (ptr-dt), i++) {
+    char* comma_ptr = strstr(ptr, ",");
+    if (comma_ptr == NULL) {
+      printBug("get_devtree_compatible_struct: Unable to find comma (num_vendors=%d)", num_vendors);
+      return NULL;
+    }
+    comma_ptr = comma_ptr-1; // Point right before comma
+
+    char* end_ptr = memchr(comma_ptr, '\0', len - ptrpos);
+    if (end_ptr == NULL) {
+      printBug("get_devtree_compatible_struct: Unable to find delimiter (2) (num_vendors=%d)", num_vendors);
+      return NULL;
+    }
+
+    int vendor_str_len = (comma_ptr-ptr)+1;
+    int model_str_len = (end_ptr-(comma_ptr+2))+1;
+
+    vendors[i] = emalloc(sizeof(struct devtree));
+    vendors[i]->vendor = ecalloc(vendor_str_len, sizeof(char));
+    vendors[i]->model = ecalloc(model_str_len, sizeof(char));
+
+    strncpy(vendors[i]->vendor, ptr, vendor_str_len);
+    strncpy(vendors[i]->model, comma_ptr+2, model_str_len);
+
+    ptr = memchr(ptr, '\0', len);
+    if (ptr == NULL) {
+      printBug("get_devtree_compatible_struct: Unable to find delimiter (3) (num_vendors=%d)", num_vendors);
+      return NULL;
+    }
+    ptr++; // Point right after delimiter
+  }
+
+  *num_vendors_ptr = num_vendors;
+  return vendors;
+}
